//+------------------------------------------------------------------+
//|                                              GannTrendSignal.mqh |
//|                                                 Dima S., 2010 . |
//|                                                 dimascub@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Dima S., 2010 ."
#property link      "dimascub@mail.ru"
//---------------------------------------------------------------------
#include	<Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=               |
//|    ( iCustom )                                   |
//| Type=Signal                                                      |
//| Name=TGannBreakSignal                                            |
//| Class=TGannBreakSignal                                           |
//| Page=                                                            |
//| Parameter=MinMainSwingContinuance,int,5                          |
//| Parameter=MinMainSwingSize,double,300.0                          |
//| Parameter=MinMiddleSwingContinuance,int,3                        |
//| Parameter=MaxMiddleSwingSize,double,200.0                        |
//| Parameter=OpenPriceSpace,double,5.0                             |
//| Parameter=StopLossSpace,double,5.0                              |
//+------------------------------------------------------------------+
// wizard description end
//=====================================================================
//	  BUY/SELL    :
//=====================================================================
class TGannBreakSignal : public CExpertSignal
{
private:
	int				min_main_swing_continuance;																//       
	double		min_main_swing_size_points;																//       
	int				min_middle_swing_continuance;															//        
	double		max_middle_swing_size_points;															//       
	double		open_price_space;																					//   /    
	double		stop_loss_space;																					//   /    -

private:
	datetime	main_swing_lf_datetime;																		//         
	double		main_swing_lf_price;																			//         
	datetime	main_swing_rt_datetime;																		//         
	double		main_swing_rt_price;																			//         
	int				main_swing_continuance;																		//      
	double		main_swing_size_points;																		//      

private:
	datetime	middle_swing_lf_datetime;																	//         
	double		middle_swing_lf_price;																		//         
	datetime	middle_swing_rt_datetime;																	//         
	double		middle_swing_rt_price;																		//        
	int				middle_swing_continuance;																	//       
	double		middle_swing_size_points;																	//      

private:
	int				handle_main_swing;
	int				handle_middle_swing;
	double		main_swing_buff[ ];
	double		middle_swing_buff[ ];
	datetime	time_buff[ ];
	double		price_buff[ ];

public:
	TGannBreakSignal( );																								// 
	~TGannBreakSignal( );																								// 

public:
//	 :
	void		MinMainSwingContinuance( int _cont );
	void		MinMainSwingSize( double _size );
	void		MinMiddleSwingContinuance( int _cont );
	void		MaxMiddleSwingSize( double _size );
	void		OpenPriceSpace( double _space );
	void		StopLossSpace( double _space );

public:
	int			GetMainSwingContinuance( );																	//        
	double	GetMainSwingSizePoints( );																	//      4-     
	int			GetMiddleSwingContinuance( );																//        
	double	GetMiddleSwingSizePoints( );																//      4-     

//	   :
	virtual bool	ValidationSettings( );
	virtual bool	CheckOpenLong( double& price, double& sl, double& tp, datetime& expiration );
	virtual bool	CheckOpenShort( double& price, double& sl, double& tp, datetime& expiration );
	virtual bool	InitIndicators( CIndicators* indicators );

//	 :
protected:
//	     :
	void		SetMainSwingParameters( datetime _lf_dt, double _lf_price, datetime _rt_dt, double _rt_price );
//	     :
	void		SetMiddleSwingParameters( datetime _lf_dt, double _lf_price, datetime _rt_dt, double _rt_price );
	int			GetMainSwing( );																						//       
	int			GetMiddleSwing( );																					//       
};
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TGannBreakSignal::TGannBreakSignal( )
{
	this.min_main_swing_continuance = 0;
	this.min_main_swing_size_points = 0.0;
	this.min_middle_swing_continuance = 0;
	this.max_middle_swing_size_points = 0.0;
	this.open_price_space = 0.0;
	this.stop_loss_space = 0.0;
}

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TGannBreakSignal::~TGannBreakSignal( )
{
	if( this.handle_main_swing != INVALID_HANDLE )
	{
		IndicatorRelease( this.handle_main_swing );
	}
	if( this.handle_middle_swing != INVALID_HANDLE )
	{
		IndicatorRelease( this.handle_middle_swing );
	}
}

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------
void		TGannBreakSignal::MinMainSwingContinuance( int _cont )
{
	this.min_main_swing_continuance = _cont;
}
void		TGannBreakSignal::MinMainSwingSize( double _size )
{
	this.min_main_swing_size_points = _size;
}
void		TGannBreakSignal::MinMiddleSwingContinuance( int _cont )
{
	this.min_middle_swing_continuance = _cont;
}
void		TGannBreakSignal::MaxMiddleSwingSize( double _size )
{
	this.max_middle_swing_size_points = _size;
}
void		TGannBreakSignal::OpenPriceSpace( double _space )
{
	this.open_price_space = _space;
}
void		TGannBreakSignal::StopLossSpace( double _space )
{
	this.stop_loss_space = _space;
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
bool		TGannBreakSignal::ValidationSettings( )
{
	if( this.min_main_swing_continuance <= 0 )
	{
		Print( "Wrong Parameter: min_main_swing_continuance = ", this.min_main_swing_continuance );
		return( false );
	}
	if( this.min_main_swing_size_points <= 0.0 )
	{
		Print( "Wrong Parameter: min_main_swing_size_points = ", DoubleToString( this.min_main_swing_size_points, 1 ));
		return( false );
	}
	if( this.min_middle_swing_continuance <= 0 )
	{
		Print( "Wrong Parameter: min_middle_swing_continuance = ", this.min_middle_swing_continuance );
		return( false );
	}
	if( this.max_middle_swing_size_points <= 0.0 )
	{
		Print( "Wrong Parameter: max_middle_swing_size_points = ", DoubleToString( this.max_middle_swing_size_points, 1 ));
		return( false );
	}

	return( true );
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
bool		TGannBreakSignal::CheckOpenLong( double& _price, double& _sl, double& _tp, datetime& _expiration )
{
	if( this.GetMainSwing( ) == -1 )
	{
		return( false );
	}
	
	if( this.GetMiddleSwing( ) == -1 )
	{
		return( false );
	}

//	   ,       :
	if( this.main_swing_rt_price >= this.main_swing_lf_price )
	{
		return( false );
	}

//	  ""    ,       :
	if( this.middle_swing_rt_price >= this.middle_swing_lf_price )
	{
		return( false );
	}

//	      :
	if( this.main_swing_continuance < this.min_main_swing_continuance || this.main_swing_size_points < this.min_main_swing_size_points )
	{
		return( false );
	}

//	      :
	if( this.middle_swing_continuance < this.min_middle_swing_continuance || this.middle_swing_size_points > this.max_middle_swing_size_points )
	{
		return( false );
	}

	double	unit = this.PriceLevelUnit( );

//	    ""  ,  :
	double	delta = this.m_symbol.Bid( ) - ( this.middle_swing_lf_price + this.open_price_space * unit );
	if(( delta >= 0.0 ) && ( delta < ( 10.0 * unit )))
	{
		_price = 0.0;
		_sl = this.m_symbol.NormalizePrice( this.middle_swing_rt_price - stop_loss_space * unit );
		_tp = 0.0;

		return( true );
	}

	return( false );
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
bool		TGannBreakSignal::CheckOpenShort( double& _price, double& _sl, double& _tp, datetime& _expiration )
{
	if( this.GetMainSwing( ) == -1 )
	{
		return( false );
	}
	
	if( this.GetMiddleSwing( ) == -1 )
	{
		return( false );
	}

//	   ,       :
	if( this.main_swing_rt_price <= this.main_swing_lf_price )
	{
		return( false );
	}

//	  ""    ,       :
	if( this.middle_swing_rt_price <= this.middle_swing_lf_price )
	{
		return( false );
	}

//	      :
	if( this.main_swing_continuance < this.min_main_swing_continuance || this.main_swing_size_points < this.min_main_swing_size_points )
	{
		return( false );
	}

//	      :
	if( this.middle_swing_continuance < this.min_middle_swing_continuance || this.middle_swing_size_points > this.max_middle_swing_size_points )
	{
		return( false );
	}

	double	unit = this.PriceLevelUnit( );

//	    ""  ,  :
	double	delta = ( this.middle_swing_lf_price - this.open_price_space * unit ) - this.m_symbol.Bid( );
	if(( delta >= 0.0 ) && ( delta < ( 10.0 * unit )))
	{
		_price = 0.0;
		_sl = this.m_symbol.NormalizePrice( this.middle_swing_rt_price + stop_loss_space * unit );
		_tp = 0.0;

		return( true );
	}

	return( false );
}

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
bool		TGannBreakSignal::InitIndicators( CIndicators* _ind )
{
	this.handle_main_swing = iCustom( this.m_symbol.Name( ), this.m_period, "GannMainTrend", 1000, false, false, 1, 1, LightCyan, LightGreen, LightPink, LightGreen, LightPink, 3 );
	if( this.handle_main_swing == INVALID_HANDLE )
	{
		return( false );
	}

	this.handle_middle_swing = iCustom( this.m_symbol.Name( ), this.m_period, "GannMiddleTrend", 1000, false, false, 1, 1, LightCyan, LightGreen, LightPink, LightGreen, LightPink, 3 );
	if( this.handle_middle_swing == INVALID_HANDLE )
	{
		return( false );
	}

	return( true );
}

//---------------------------------------------------------------------
//	      :
//---------------------------------------------------------------------
#define	SIZE_MAIN		200
//---------------------------------------------------------------------
int			TGannBreakSignal::GetMainSwing( )
{
	bool	is_high_found = false, is_low_found = false;
	int		bars_high = 0, bars_low = 0;

//	   :
	if( CopyBuffer( this.handle_main_swing, 1, 0, SIZE_MAIN, this.main_swing_buff ) != SIZE_MAIN )
	{
		return( -1 );
	}

//	   :
	if( CopyBuffer( this.handle_main_swing, 0, 0, SIZE_MAIN, this.price_buff ) != SIZE_MAIN )
	{
		return( -1 );
	}

//	     :
	int 	index;
	bool	is_find = false;
	for( index = SIZE_MAIN - 1; index >= 0; index-- )
	{
		if( this.price_buff[ index ] > 0.1 )
		{
			is_find = true;
			break;
		}
	}
	if( is_find == false )
	{
		return( -1 );
	}

//	      :
	int		color_prev = ( int )( this.main_swing_buff[ index ] );
	for( int i = index - 1; i >= 0; i-- )
	{
		if( is_high_found == false && this.price_buff[ i ] > 0.1 && this.main_swing_buff[ i ] == 0.0 && color_prev == 1 )				//  
		{
			is_high_found = true;
			bars_high = i;

//	    ,    :
			if( is_low_found == true )
			{
				break;
			}

			color_prev = ( int )( this.main_swing_buff[ i ] );
		}
		else if( is_low_found == false && this.price_buff[ i ] > 0.1 && this.main_swing_buff[ i ] == 1.0 && color_prev == 0 )		//  
		{
			is_low_found = true;
			bars_low = i;
		}

//	    ,    :
		if( is_high_found == true )
		{
			break;
		}

		if( this.price_buff[ i ] > 0.1 )
		{
			color_prev = ( int )( this.main_swing_buff[ i ] );
		}
	}

//	     ,    :
	if( is_high_found == true && is_low_found == true )
	{
//	 :
		if( CopyTime( this.m_symbol.Name( ), this.m_period, 0, SIZE_MAIN, this.time_buff ) != SIZE_MAIN )
		{
			return( -1 );
		}
//	  /:		
		if( bars_low > bars_high )
		{
			this.main_swing_lf_datetime = this.time_buff[ bars_high ];			//         
			this.main_swing_lf_price = this.price_buff[ bars_high ];				//         
			this.main_swing_rt_datetime = this.time_buff[ bars_low ];				//         
			this.main_swing_rt_price = this.price_buff[ bars_low ];					//         
		}
		else if( bars_high > bars_low )
		{
			this.main_swing_lf_datetime = this.time_buff[ bars_low ];				//         
			this.main_swing_lf_price = this.price_buff[ bars_low ];					//         
			this.main_swing_rt_datetime = this.time_buff[ bars_high ];			//         
			this.main_swing_rt_price = this.price_buff[ bars_high ];					//         
		}
		else
		{
			return( -1 );
		}

//	  :
		this.SetMainSwingParameters( this.main_swing_lf_datetime, this.main_swing_lf_price, this.main_swing_rt_datetime, this.main_swing_rt_price );

		return( 0 );
	}
	else
	{
		return( -1 );
	}
}

//---------------------------------------------------------------------
//	      :
//---------------------------------------------------------------------
#define	SIZE_MIDDLE		200
//---------------------------------------------------------------------
int			TGannBreakSignal::GetMiddleSwing( )
{
	bool	is_high_found = false, is_low_found = false;
	int		bars_high = 0, bars_low = 0;

//	   :
	if( CopyBuffer( this.handle_middle_swing, 1, 0, SIZE_MIDDLE, this.middle_swing_buff ) != SIZE_MIDDLE )
	{
		return( -1 );
	}

//	   :
	if( CopyBuffer( this.handle_middle_swing, 0, 0, SIZE_MIDDLE, this.price_buff ) != SIZE_MIDDLE )
	{
		return( -1 );
	}

//	     :
	int 	index;
	bool	is_find = false;
	for( index = SIZE_MIDDLE - 1; index >= 0; index-- )
	{
		if( this.price_buff[ index ] > 0.1 )
		{
			is_find = true;
			break;
		}
	}
	if( is_find == false )
	{
		return( -1 );
	}

//	      :
	int		color_prev = ( int )( this.middle_swing_buff[ index ] );
	for( int i = index - 1; i >= 0; i-- )
	{
		if( is_high_found == false && this.price_buff[ i ] > 0.1 && this.middle_swing_buff[ i ] == 0.0 && color_prev == 1 )				//  
		{
			is_high_found = true;
			bars_high = i;

//	    ,    :
			if( is_low_found == true )
			{
				break;
			}

			color_prev = ( int )( this.middle_swing_buff[ i ] );
		}
		else if( is_low_found == false && this.price_buff[ i ] > 0.1 && this.middle_swing_buff[ i ] == 1.0 && color_prev == 0 )		//  
		{
			is_low_found = true;
			bars_low = i;
		}

//	    ,    :
		if( is_high_found == true )
		{
			break;
		}

		if( this.price_buff[ i ] > 0.1 )
		{
			color_prev = ( int )( this.middle_swing_buff[ i ] );
		}
	}

//	     ,    :
	if( is_high_found == true && is_low_found == true )
	{
//	 :
		if( CopyTime( this.m_symbol.Name( ), this.m_period, 0, SIZE_MIDDLE, this.time_buff ) != SIZE_MIDDLE )
		{
			return( -1 );
		}

//	  /:		
		if( bars_low > bars_high )
		{
			this.middle_swing_lf_datetime = this.time_buff[ bars_high ];		//         
			this.middle_swing_lf_price = this.price_buff[ bars_high ];			//         
			this.middle_swing_rt_datetime = this.time_buff[ bars_low ];			//         
			this.middle_swing_rt_price = this.price_buff[ bars_low ];				//         
		}
		else if( bars_high > bars_low )
		{
			this.middle_swing_lf_datetime = this.time_buff[ bars_low ];			//         
			this.middle_swing_lf_price = this.price_buff[ bars_low ];				//         
			this.middle_swing_rt_datetime = this.time_buff[ bars_high ];		//         
			this.middle_swing_rt_price = this.price_buff[ bars_high ];			//         
		}
		else
		{
			return( -1 );
		}

//	  :
		this.SetMiddleSwingParameters( this.middle_swing_lf_datetime, this.middle_swing_lf_price, this.middle_swing_rt_datetime, this.middle_swing_rt_price );

		return( 0 );
	}
	else
	{
		return( -1 );
	}
}

//---------------------------------------------------------------------
//	     :
//---------------------------------------------------------------------
void		TGannBreakSignal::SetMainSwingParameters( datetime _lf_dt, double _lf_price, datetime _rt_dt, double _rt_price )
{
	this.main_swing_continuance = ( int )( _rt_dt - _lf_dt ) / PeriodSeconds( this.m_period );
	this.main_swing_size_points = MathAbs( _rt_price - _lf_price ) / this.PriceLevelUnit( );
}

//---------------------------------------------------------------------
//	     :
//---------------------------------------------------------------------
void		TGannBreakSignal::SetMiddleSwingParameters( datetime _lf_dt, double _lf_price, datetime _rt_dt, double _rt_price )
{
	this.middle_swing_continuance = ( int )( _rt_dt - _lf_dt ) / PeriodSeconds( this.m_period );
	this.middle_swing_size_points = MathAbs( _rt_price - _lf_price ) / this.PriceLevelUnit( );
}

//---------------------------------------------------------------------
//	       :
//---------------------------------------------------------------------
int			TGannBreakSignal::GetMainSwingContinuance( )
{
	return( this.main_swing_continuance );
}

//---------------------------------------------------------------------
//	     4-     :
//---------------------------------------------------------------------
double	TGannBreakSignal::GetMainSwingSizePoints( )
{
	return( this.main_swing_size_points );
}

//---------------------------------------------------------------------
//	       :
//---------------------------------------------------------------------
int			TGannBreakSignal::GetMiddleSwingContinuance( )
{
	return( this.middle_swing_continuance );
}

//---------------------------------------------------------------------
//	     4-     :
//---------------------------------------------------------------------
double	TGannBreakSignal::GetMiddleSwingSizePoints( )
{
	return( this.middle_swing_size_points );
}
//---------------------------------------------------------------------
